#ifdef VIDEOSTATUS
#include <sys/ioctl.h>
#endif

#if VDRVERSNUM >= 10332
#include <vdr/device.h>
#endif

#include "DisplayChannel.h"
#include "vdrstatus.h"
#include "videostatus.h"
#include <vdr/menu.h>

#include "symbols/audio.xpm"
#include "symbols/dolbydigital.xpm"
#include "symbols/encrypted.xpm"
#include "symbols/radio.xpm"
#include "symbols/recording.xpm"
#include "symbols/teletext.xpm"
#include "symbols/vps.xpm"

#ifdef VIDEOSTATUS
#include "symbols/test/ar16943.xpm"
#include "symbols/test/ar169na.xpm"
#include "symbols/test/ar169ps.xpm"
#include "symbols/test/ar169vc.xpm"
#include "symbols/test/ar43169ps.xpm"
#include "symbols/test/ar43169vc.xpm"
#include "symbols/test/ar432211ps.xpm"
#include "symbols/test/ar432211psvc.xpm"
#include "symbols/test/ar432211vc.xpm"
#include "symbols/test/ar43na.xpm"
#endif

#define Gap          5

cBitmap cSkinElchiDisplayChannel::bmVps(vps_xpm);
cBitmap cSkinElchiDisplayChannel::bmTeletext(teletext_xpm);
cBitmap cSkinElchiDisplayChannel::bmRadio(radio_xpm);
cBitmap cSkinElchiDisplayChannel::bmAudio(audio_xpm);
cBitmap cSkinElchiDisplayChannel::bmDolbyDigital(dolbydigital_xpm);
cBitmap cSkinElchiDisplayChannel::bmEncrypted(encrypted_xpm);
cBitmap cSkinElchiDisplayChannel::bmRecording(recording_xpm);

#ifdef VIDEOSTATUS
cBitmap cSkinElchiDisplayChannel::bmar16943(ar16943_xpm);
cBitmap cSkinElchiDisplayChannel::bmar169na(ar169na_xpm);
cBitmap cSkinElchiDisplayChannel::bmar169ps(ar169ps_xpm);
cBitmap cSkinElchiDisplayChannel::bmar169vc(ar169vc_xpm);
cBitmap cSkinElchiDisplayChannel::bmar43169ps(ar43169ps_xpm);
cBitmap cSkinElchiDisplayChannel::bmar43169vc(ar43169vc_xpm);
cBitmap cSkinElchiDisplayChannel::bmar432211ps(ar432211ps_xpm);
cBitmap cSkinElchiDisplayChannel::bmar432211psvc(ar432211psvc_xpm);
cBitmap cSkinElchiDisplayChannel::bmar432211vc(ar432211vc_xpm);
cBitmap cSkinElchiDisplayChannel::bmar43na(ar43na_xpm);

#include <linux/dvb/video.h>
extern video_status videostatus;
extern int fd_video;
extern video_size_t vs;
bool video = true;
#define DEV_DVB_VIDEO     "/dev/dvb/adapter%d/video%d"
#endif

// --- cSkinElchiDisplayChannel --------------------------------------------

cSkinElchiDisplayChannel::cSkinElchiDisplayChannel(bool WithInfo)
{
#ifdef DEBUG
  isyslog("skinelchi: cSkinElchiDisplayChannel::cSkinElchiDisplayChannel(%s)", WithInfo ? "'WithInfo'" : "'WithoutInfo'");
#endif
  noflush = true;
  message = changed = false;
  volumechange = VdrStatus.VolumeChange;
  asprintf(&olddate, " ");
  oldaudiow = -1;
  rec = false;
#ifdef VIDEOSTATUS
  asprintf(&oldvideoformatchar, " ");
  oldvideow = -1;
  if (SkinElchiSetup.videostatus && cDevice::PrimaryDevice()->HasDecoder()) {
    if (fd_video != -1) {
        close(fd_video);
        fd_video = -1;
        }
    char *dev = NULL;
    asprintf(&dev, DEV_DVB_VIDEO, cDevice::PrimaryDevice()->CardIndex(), 0);
    fd_video = open(dev, O_RDONLY | O_NONBLOCK);
    oldbm = NULL;
    free(dev);
    }
#endif
  present = NULL;
  track = NULL;
  oldpresentvps = -1;
  lastSeen = -1;
  withInfo = WithInfo;

  smallfont = cFont::GetFont(fontSml);
  font = cFont::GetFont(fontOsd);
  lh = font->Height();
  slh = smallfont->Height();
  lhh = lh / 2;
  lineOffset = (lh - slh) / 2;
  bg = Theme.Color(clrBackground);
  if (SkinElchiSetup.logo) {
    logox = 64;
    logoy = 48;
    }
  else
    logox = logoy = 0;

  x0 = 0;
  x1 = SkinElchiSetup.logo ? 16 : 0;
  x2 = x1 + lh - lhh;
  x3 = logox + lh - lhh;
  x4 = x2 + font->Width("00:00");
  x5 = x4 + lhh + Gap;
  x6 = x3 + font->Width("00000-") + smallfont->Width("_000 00.00 00:00");
  x7 = x6 + lh - lhh;
  x11 = Setup.OSDWidth;
  x8 = x11 - lh;
  x10 = x8 + lhh;
  x9 = x10 - lh - bmRecording.Width() - 3 - bmEncrypted.Width() - 3 - bmDolbyDigital.Width() - 3 - bmAudio.Width() - 3 - bmTeletext.Width() - 3 - bmVps.Width() - 3;
#ifdef VIDEOSTATUS
  if (fd_video >= 0)
    x9 -= bmar16943.Width() + 3;
#endif
  y0 = 0;
  y1 = SkinElchiSetup.logo ? 16 : 0;
  y3 = y1 + lh;
  y2 = y3 - lhh;
  y4 = y3 + 3;
  y6 = y4 + lh;
  y5 = y6 - lhh;
  y9 = y6 + (withInfo ? lh * 4 : 0);
  y7 = y9 - lh;
  y8 = y9 - lhh;

  bool OSDtoBIGforNTSC = cDevice::PrimaryDevice()->HasDecoder() && (cDevice::PrimaryDevice()->GetVideoSystem() == 1) && ((Setup.OSDTop + Setup.OSDHeight) > 480);
  osd = cOsdProvider::NewOsd(Setup.OSDLeft, OSDtoBIGforNTSC ? (Setup.ChannelInfoPos ? 0 : 480 - y9) : (Setup.OSDTop + (Setup.ChannelInfoPos ? 0 : Setup.OSDHeight - y9)));

  if (withInfo) {
    tArea Areas[] = { { 0, 0, x11 - 1, y9 - 1, 8 } };
    if ((osd->CanHandleAreas(Areas, sizeof(Areas) / sizeof(tArea)) == oeOk) && (SkinElchiSetup.singlearea)) {
    	osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
        if (SkinElchiSetup.logo) {
	    for (int i = 0; i < MAX_ELCHI_THEME_COLORS; i++) {
		osd->GetBitmap(0)->SetColor(i,SkinElchiSetup.ThemeColors[i]);
		}
    	    osd->DrawRectangle(logox, y0, x11 - 1, y1 - 1, clrTransparent);
    	    osd->DrawRectangle(x0, logoy, x1 - 1, y9 - 1, clrTransparent);
    	    }
        }
    else {
        if (SkinElchiSetup.logo) {
    	    tArea Areas[] = {{ logox, y1, x11 - 1, y6 - 1, 4 },
			    { x1, logoy, logox - 1, y6 - 1, 4 }, 
			    { x0, y0, logox - 1, logoy - 1, 8 },
			    { x1, y6, x11 - 1, y9 - 1, 4 },};
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
        }
	else {
    	    tArea Areas[] = { { x0, y0, x11 - 1, y6 - 1, 4 },
			    { x0, y6, x11 - 1, y9 - 1, 4 }, };
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
	}
    }

    if (SkinElchiSetup.logo) 
	osd->DrawRectangle(logox, y1, x3 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
    else {
	osd->DrawRectangle(logox, y1, x3 - 1, y2 - 1, clrTransparent);
	osd->DrawEllipse  (logox, y1, x3 - 1, y2 - 1, Theme.Color(clrChannelDateBg), 2);
	osd->DrawRectangle(logox, y2, x3 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
	}

    osd->DrawRectangle(x3, y1, x6 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
    osd->DrawRectangle(x6, y1, x10 - 1, y3 - 1, bg);
    osd->DrawEllipse  (x6, y1, x7 - 1, y3 - 1, Theme.Color(clrChannelDateBg), 5);

    osd->DrawRectangle(x10, y1, x11 - 1, y2 - 1, clrTransparent);
    osd->DrawEllipse  (x10, y1, x11 - 1, y2 - 1, bg, 1);
    osd->DrawRectangle(x10, y2, x11 - 1, y3 - 1, bg);

    osd->DrawRectangle(logox, y3, x11 - 1, y4 - 1, bg);

    if (SkinElchiSetup.logo) 
	osd->DrawRectangle(x1, logoy, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));

    osd->DrawRectangle(logox, y4, x3 - 1, y6 - 1, Theme.Color(clrChannelNameBg));

    osd->DrawRectangle(x3, y4, x9 - 1, y6 - 1, Theme.Color(clrChannelNameBg));

    osd->DrawRectangle(x9, y4, x11 - 1, y6 - 1, bg);
    osd->DrawEllipse  (x9, y4, x9 + lhh - 1, y6 - 1, Theme.Color(clrChannelNameBg), 5);

    osd->DrawRectangle(x1, y6, x4 - 1, y8 - 1, Theme.Color(clrChannelEpgTimeBg));
    osd->DrawRectangle(x4, y6, x11 - 1, y8 - 1, Theme.Color(clrChannelEpgTitleBg));
  
    osd->DrawRectangle(x1, y8, x2 - 1, y9 - 1, clrTransparent);
    osd->DrawEllipse  (x1, y8, x2 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg), 3);

    osd->DrawRectangle(x2, y8, x4 - 1, y9 - 1, Theme.Color(clrChannelEpgTimeBg));
    osd->DrawRectangle(x4, y8, x10 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg));

    osd->DrawRectangle(x10, y8, x11 - 1, y9 - 1, clrTransparent);
    osd->DrawEllipse  (x10, y8, x11 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg), 4);

    EventScroll.DestWidth = x10 - x5 - 2;
    EventScroll.Fg = Theme.Color(clrChannelEpgTitle);
    EventScroll.Bg = Theme.Color(clrChannelEpgTitleBg);
    EventScroll.Font = font;
    EventScroll.xOff = x5 + 2;
    EventScroll.yOff = y6;

    EventShortScroll.DestWidth = EventScroll.DestWidth;
    EventShortScroll.Fg = Theme.Color(clrChannelEpgShortText);
    EventShortScroll.Bg = Theme.Color(clrChannelEpgTitleBg);
    EventShortScroll.Font = smallfont;
    EventShortScroll.xOff = EventScroll.xOff;
    EventShortScroll.yOff = y6 + lh;

    NextEventScroll.DestWidth = EventScroll.DestWidth;
    NextEventScroll.Fg = Theme.Color(clrChannelEpgTitle);
    NextEventScroll.Bg = Theme.Color(clrChannelEpgTitleBg);
    NextEventScroll.Font = font;
    NextEventScroll.xOff = EventScroll.xOff;
    NextEventScroll.yOff = EventShortScroll.yOff + lh;

    NextEventShortScroll.DestWidth = EventScroll.DestWidth;
    NextEventShortScroll.Fg = Theme.Color(clrChannelEpgShortText);
    NextEventShortScroll.Bg = Theme.Color(clrChannelEpgTitleBg);
    NextEventShortScroll.Font = smallfont;
    NextEventShortScroll.xOff = EventScroll.xOff;
    NextEventShortScroll.yOff = NextEventScroll.yOff + lh;
    }
  else {
    tArea Areas[] = { { 0, 0, x11 - 1, y9 - 1, 8 } };
    if (osd->CanHandleAreas(Areas, sizeof(Areas) / sizeof(tArea)) == oeOk) {
    	osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
        if (SkinElchiSetup.logo) {
	    for (int i = 0; i < MAX_ELCHI_THEME_COLORS; i++) {
		osd->GetBitmap(0)->SetColor(i,SkinElchiSetup.ThemeColors[i]);
		}
    	    osd->DrawRectangle(logox, y0, x11 - 1, y1 - 1, clrTransparent);
    	    osd->DrawRectangle(x0, logoy, x1 - 1, y9 - 1, clrTransparent);
    	    }
        }
    else {
	if (SkinElchiSetup.logo) {
    	    tArea Areas[] ={{ logox, y1, x11 - 1, y9 - 1, 4 },
			    { x1, logoy, logox - 1, y9 - 1, 4 }, 
			    { x0, y0, logox - 1, logoy - 1, 8 },};
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));

    	    osd->DrawRectangle(logox, y0, x11 - 1, y1 - 1, clrTransparent);
    	    osd->DrawRectangle(x0, logoy, x1 - 1, y9 - 1, clrTransparent);
        }
	else {
    	    tArea Areas[] = { { x0, y0, x11 - 1, y9 - 1, 4 } };
    	    osd->SetAreas(Areas, sizeof(Areas) / sizeof(tArea));
	}
    }

    if (SkinElchiSetup.logo) {
        osd->DrawRectangle(logox, y1, x3 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
        }
    else {
        osd->DrawRectangle(logox, y1, x3 - 1, y2 - 1, clrTransparent);
        osd->DrawEllipse  (logox, y1, x3 - 1, y2 - 1, Theme.Color(clrChannelDateBg), 2);
        osd->DrawRectangle(logox, y2, x3 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
    }
    osd->DrawRectangle(x3, y1, x6 - 1, y3 - 1, Theme.Color(clrChannelDateBg));
    osd->DrawRectangle(x6, y1, x10 - 1, y3 - 1, bg);
    osd->DrawEllipse  (x6, y1, x7 - 1, y3 - 1, Theme.Color(clrChannelDateBg), 5);

    osd->DrawRectangle(x10, y1, x11 - 1, y2 - 1, clrTransparent);
    osd->DrawEllipse  (x10, y1, x11 - 1, y2 - 1, bg, 1);
    osd->DrawRectangle(x10, y2, x11 - 1, y3 - 1, bg);

    osd->DrawRectangle(logox, y3, x11 - 1, y4 - 1, bg);

    if (SkinElchiSetup.logo) {
	osd->DrawRectangle(x1, logoy, logox - 1, y5 - 1, Theme.Color(clrChannelNameBg));
        osd->DrawRectangle(x2, y5, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));
        osd->DrawRectangle(logox, y4, x3 - 1, y6 - 1, Theme.Color(clrChannelNameBg));
	}
    else
        osd->DrawRectangle(logox, y4, x3 - 1, y5 - 1, Theme.Color(clrChannelNameBg));

    osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
    osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 3);

    osd->DrawRectangle(x3, y4, x9 - 1, y6 - 1, Theme.Color(clrChannelNameBg));

    osd->DrawRectangle(x9, y4, x11 - 1, y6 - 1, bg);
    osd->DrawEllipse  (x9, y4, x9 + lhh - 1, y6 - 1, Theme.Color(clrChannelNameBg), 5);

    osd->DrawRectangle(x9 + lhh, y4, x11 - 1, y5 - 1, bg);
    osd->DrawRectangle(x9 + lhh, y5, x10 - 1, y6 - 1, bg);

    osd->DrawRectangle(x10, y5, x11 - 1, y6 - 1, clrTransparent);
    osd->DrawEllipse  (x10, y5, x11 - 1, y6 - 1, bg, 4);
    }

    if (SkinElchiSetup.logo) {
        osd->DrawRectangle(x0, y0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
        osd->DrawRectangle(logox, y1/*0 + 2*/, logox + 2 - 1, logoy + 2 - 1, bg);
        osd->DrawRectangle(x1, logoy, logox - 1, logoy + 2 - 1, bg);
    }
    //for scrolling
    AudioStringScroll.DestWidth = x10 - x7;
    AudioStringScroll.Fg = Theme.Color(clrChannelSymbolOn);
    AudioStringScroll.Bg = bg;
    AudioStringScroll.Font = smallfont;
    AudioStringScroll.xOff = x7;
    AudioStringScroll.yOff = y1 + lineOffset;

    ChannelNameScroll.DestWidth = x9 -
#ifdef VIDEOSTATUS
((fd_video >= 0) ? smallfont->Width(" 0000x0000 (2:2:2:2:2)") : 0) - 
#endif
x3;
    ChannelNameScroll.Fg = Theme.Color(clrChannelNameFg);
    ChannelNameScroll.Bg = Theme.Color(clrChannelNameBg);
    ChannelNameScroll.Font = smallfont;
    ChannelNameScroll.xOff = x3;
    ChannelNameScroll.yOff = y4 + lineOffset;
    noflush = false;
}

cSkinElchiDisplayChannel::~cSkinElchiDisplayChannel()
{
#ifdef DEBUG
    isyslog("skinelchi: cSkinElchiDisplayChannel::~cSkinElchiDisplayChannel()");
#endif
    noflush = true;
    message = changed = false;
    if (olddate != NULL)
	free(olddate);
#ifdef VIDEOSTATUS
    if (oldvideoformatchar != NULL)
	free(oldvideoformatchar);
    if (fd_video != -1 && !SkinElchiSetup.videostatus) {
	close(fd_video);
	fd_video = -1;
	}
#endif
    delete osd;
}

void cSkinElchiDisplayChannel::SetChannel(const cChannel *Channel, int Number)
{
#ifdef DEBUG
  isyslog("skinelchi: cSkinElchiDisplayChannel::SetChannel(...,%d)", Number);
#endif
  if (AudioStringScroll.Text != NULL)
    free(AudioStringScroll.Text);
  asprintf(&AudioStringScroll.Text, " ");
#ifdef VIDEOSTATUS
  if (oldvideoformatchar != NULL)
    free(oldvideoformatchar);
  asprintf(&oldvideoformatchar, " ");
#endif
  changed = true;
  oldpresentvps = -1;
  osd->DrawRectangle(x9 + lh / 2, y4, x10 - 1, y6 - 1, bg);
  int x = x10;
  if (Channel) {
    if (!Channel->GroupSep()) {
	int d = 3;
	x -= bmRecording.Width();
	osd->DrawBitmap(x, y4 + (y6 - y4 - bmRecording.Height()) / 2, bmRecording, Theme.Color(rec ? clrChannelSymbolRecFg : clrChannelSymbolOff), rec ? Theme.Color(clrChannelSymbolRecBg) : bg);
        x -= bmEncrypted.Width() + d;
        osd->DrawBitmap(x, y4 + (y6 - y4 - bmEncrypted.Height()) / 2, bmEncrypted, Theme.Color(Channel->Ca() ? clrChannelSymbolOn : clrChannelSymbolOff), bg);
	x -= bmDolbyDigital.Width() + d;
	osd->DrawBitmap(x, y4 + (y6 - y4 - bmDolbyDigital.Height()) / 2, bmDolbyDigital, Theme.Color(Channel->Dpid(0) ? clrChannelSymbolOn : clrChannelSymbolOff), bg);
        x -= bmAudio.Width() + d;
        osd->DrawBitmap(x, y4 + (y6 - y4 - bmAudio.Height()) / 2, bmAudio, Theme.Color(Channel->Apid(1) ? clrChannelSymbolOn : clrChannelSymbolOff), bg);
	if (Channel->Vpid()) {
    	    x -= bmTeletext.Width() + d;
    	    osd->DrawBitmap(x, y4 + (y6 - y4 - bmTeletext.Height()) / 2, bmTeletext, Theme.Color(Channel->Tpid() ? clrChannelSymbolOn : clrChannelSymbolOff), bg);
#ifdef VIDEOSTATUS
	    video = true;
#endif
	    }
	else
#ifdef VIDEOSTATUS
	     {
	    video = false;
#endif
	    if (Channel->Apid(0)) {
#ifdef VIDEOSTATUS
    		x -= bmTeletext.Width() + d;
		if (Channel->Tpid())
    		    osd->DrawBitmap(x, y4 + (y6 - y4 - bmTeletext.Height()) / 2, bmTeletext, Theme.Color(clrChannelSymbolOn), bg);
		osd->DrawBitmap(x9 + lh, y4 + (y6 - y4 - bmRadio.Height()) / 2, bmRadio, Theme.Color(clrChannelSymbolOn), bg);
#else
    	        x -= bmRadio.Width() + d;
    	        osd->DrawBitmap(x, y4 + (y6 - y4 - bmRadio.Height()) / 2, bmRadio, Theme.Color(clrChannelSymbolOn), bg);
#endif
    	        }
#ifdef VIDEOSTATUS
	    }
#endif
	setchannel_x = x;
	snprintf(number, sizeof(number), "%d%s", Number ? Number : Channel->Number(), Number ? "-" : "");
        }
    else {
	snprintf(number, sizeof(number), "#");
#ifdef VIDEOSTATUS
	video = false;
#endif
	x = x10 - x7;
	osd->DrawRectangle(x10 - ((oldaudiow > x) ? x : oldaudiow), y1, x10 - 1, y3 - 1, bg);
	}

    osd->DrawText(x3, y1, number, Theme.Color(clrChannelNumberFg), Theme.Color(clrChannelDateBg), font, font->Width("00000-"));
#ifdef VIDEOSTATUS
    ChannelNameScroll.DestWidth = x9 - ((fd_video >= 0 && video) ? smallfont->Width(" 0000x0000 (2:2:2:2:2)") : 0) - x3;
#endif
    if (ChannelNameScroll.SetText(Channel->Name())) {//for scrolling
	ChannelNameScroll.pos = 0;
	ChannelNameScroll.dir = -1;
	ChannelNameScroll.delay = 5;
	ChannelNameScroll.Time.Set();
	}
    else {
	ChannelNameScroll.dir = 0;
	}
    osd->DrawText(ChannelNameScroll.xOff, ChannelNameScroll.yOff, ChannelNameScroll.Text, ChannelNameScroll.Fg, ChannelNameScroll.Bg, ChannelNameScroll.Font, ChannelNameScroll.DestWidth);
    if (SkinElchiSetup.logo) {
        if (ChannelNameScroll.Font->Width(ChannelNameScroll.Text) != 0) {
	    cBitmap bmp(logox, logoy, 8);
	    char *filename;
	    asprintf(&filename, "%s/%s%s", SkinElchiSetup.logodir, ChannelNameScroll.Text, ".xpm");
	    if (bmp.LoadXpm(filename) && (bmp.Width() == logox) && (bmp.Height() == logoy)) {
		if (osd->GetBitmap(2)) {
		    osd->GetBitmap(2)->Reset();
		}
		else {
		    osd->GetBitmap(0)->Reset();
		    for (int i = 0; i < MAX_ELCHI_THEME_COLORS; i++) {
			osd->GetBitmap(0)->SetColor(i,SkinElchiSetup.ThemeColors[i]);
			}
    		    }
		osd->DrawBitmap(0, 0, bmp);
		}
	    else
		osd->DrawRectangle(0, 0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	    free(filename);
	    }
    	else
	    osd->DrawRectangle(0, 0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	}
    }
}

void cSkinElchiDisplayChannel::SetEvents(const cEvent *Present, const cEvent *Following)
{
  if (present != Present) {
    lastSeen = -1;
    present = Present;
    }
  else if ((lastSeen < 0) || (oldpresentvps == -1)) {
    present = Present;
    }
  if (!withInfo) {
#ifdef DEBUG
  isyslog("skinelchi: skip cSkinElchiDisplayChannel::SetEvents(%d,%d)", !Present ? 0 : Present->EventID(), !Following ? 0 : Following->EventID());
#endif
    return;
    }
#ifdef DEBUG
  else
    isyslog("skinelchi: cSkinElchiDisplayChannel::SetEvents(%d,%d)", !Present ? 0 : Present->EventID(), !Following ? 0 : Following->EventID());
#endif
  changed = true;
  tColor cetb = Theme.Color(clrChannelEpgTimeBg);
  osd->DrawRectangle(x1, y6, x4 - 1, y9 - 1, cetb);
  osd->DrawRectangle(x1, y8, x2 - 1, y9 - 1, clrTransparent);
  osd->DrawEllipse  (x1, y8, x2 - 1, y9 - 1, cetb, 3);
  osd->DrawRectangle(x5, y6, x10 - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg));
    const cEvent *e = Present;
    if (e) {
        osd->DrawText(x1 + 4, y6, e->GetTimeString(), Theme.Color(clrChannelEpgTimeFg), cetb, font);
	if (EventScroll.SetText(e->Title())) {//for scrolling
	    EventScroll.pos = 0;
	    EventScroll.dir = -1;
	    EventScroll.delay = 5;
	    EventScroll.Time.Set();
	    }
	else
	    EventScroll.dir = 0;
	osd->DrawText(EventScroll.xOff, EventScroll.yOff, EventScroll.Text, EventScroll.Fg, EventScroll.Bg, EventScroll.Font, EventScroll.DestWidth);
	if (EventShortScroll.SetText(e->ShortText())) {
	    EventShortScroll.pos = 0;
	    EventShortScroll.dir = -1;
	    EventShortScroll.delay = 5;
	    EventShortScroll.Time.Set();
	    }
	else
	    EventShortScroll.dir = 0;
	osd->DrawText(EventShortScroll.xOff, EventShortScroll.yOff, EventShortScroll.Text, EventShortScroll.Fg, EventShortScroll.Bg, EventShortScroll.Font, EventShortScroll.DestWidth);
	}
    else {
    	EventShortScroll.dir = 0;
	EventScroll.dir = 0;
	}
    e = Following;
    if (e) {
        osd->DrawText(x1 + 4, y6 + 2 * lh, e->GetTimeString(), Theme.Color(clrChannelEpgTimeFg), cetb, font);
	if (NextEventScroll.SetText(e->Title())) {
	    NextEventScroll.pos = 0;
	    NextEventScroll.dir = -1;
	    NextEventScroll.delay = 5;
	    NextEventScroll.Time.Set();
	    }
	else
	    NextEventScroll.dir = 0;
	osd->DrawText(NextEventScroll.xOff, NextEventScroll.yOff, NextEventScroll.Text, NextEventScroll.Fg, NextEventScroll.Bg, NextEventScroll.Font, NextEventScroll.DestWidth);
	if (NextEventShortScroll.SetText(e->ShortText())) {
	    NextEventShortScroll.pos = 0;
	    NextEventShortScroll.dir = -1;
	    NextEventShortScroll.delay = 5;
	    NextEventShortScroll.Time.Set();
	    }
	else
	    NextEventShortScroll.dir = 0;
        osd->DrawText(NextEventShortScroll.xOff, NextEventShortScroll.yOff, NextEventShortScroll.Text, NextEventShortScroll.Fg, NextEventShortScroll.Bg, NextEventShortScroll.Font, NextEventShortScroll.DestWidth);
	}
    else {
    	NextEventShortScroll.dir = 0;
	NextEventScroll.dir = 0;
	}
}

void cSkinElchiDisplayChannel::SetMessage(eMessageType Type, const char *Text)
{
#ifdef DEBUG
  isyslog("skinelchi: cSkinElchiDisplayChannel::SetMessage(%d,%s)", (int)Type, Text);
#endif
  if (Text) {
    char *text;
    asprintf(&text, "%s", Text);
/*    if (!strcasecmp(text, tr("Going to sleep in about one minute"))) {
	asprintf(&text, "\"%s\" <- Sleeptimer-Message received -> not printed on screen", Text);
	esyslog(text);
	}
    else */{
	if (!message) {
	    osd->SaveRegion(logox, y4, x11 - 1, y6 - 1);
	    message = true;
	    }
	if (withInfo) {
	    if (SkinElchiSetup.logo)
		osd->DrawRectangle(x1, y5, logox-1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
	    osd->DrawRectangle(logox, y5, x3-1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
	    osd->DrawRectangle(x10, y4, x11 - 1, y6 - 1, bg);
	    }
	else {
	    if (SkinElchiSetup.logo) {
	        osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
	        osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type), 3);
		osd->DrawRectangle(x2, y5, logox - 1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
	    	osd->DrawRectangle(logox, y5, x3 - 1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
	    	}
	    else {
	        osd->DrawRectangle(logox, y5, x3 - 1, y6 - 1, clrTransparent);
	        osd->DrawEllipse  (logox, y5, x3 - 1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type), 3);
	        }
	    osd->DrawRectangle(x10, y5, x11 - 1, y6 - 1, clrTransparent);
	    }
	    osd->DrawRectangle(x10, y4, x11 - 1, y5 - 1, bg);
	osd->DrawEllipse(x10, y4, x11 - 1, y6 - 1, Theme.Color(clrMessageStatusBg + 2 * Type), 5);
	osd->DrawText(x3, y4, Text, Theme.Color(clrMessageStatusFg + 2 * Type), Theme.Color(clrMessageStatusBg + 2 * Type), font, x10 - x3, 0, taCenter);
	osd->DrawRectangle(logox, y4, x3-1, y5 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
	if (SkinElchiSetup.logo) {
	    osd->DrawRectangle(x0, y0, logox - 1, logoy - 1, Theme.Color(clrChannelNameBg));
	    osd->DrawRectangle(x1, logoy, logox - 1, y5 - 1, Theme.Color(clrMessageStatusBg + 2 * Type));
    	    osd->DrawRectangle(logox, y1/*0 + 2*/, logox + 2 - 1, logoy + 2 - 1, bg);
    	    osd->DrawRectangle(x1, logoy, logox - 1, logoy + 2 - 1, bg);
	    }
	changed = true;
	}
	free(text);
    }
    else {
	if (message) {
	    if (SkinElchiSetup.logo) {
         	if (withInfo)
		    osd->DrawRectangle(x1, logoy, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));
		else {
		    osd->DrawRectangle(x1, logoy, logox - 1, y5 - 1, Theme.Color(clrChannelNameBg));
    		    osd->DrawRectangle(x2, y5, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));
		    osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
		    osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 3);
		    }
    		osd->DrawRectangle(logox, y0 + 2, logox + 2 - 1, logoy + 2 - 1, bg);
    		osd->DrawRectangle(x1, logoy, logox - 1, logoy + 2 - 1, bg);
		}
	    osd->RestoreRegion();
	    changed = true;
	    message = false;
	    }
    }
}

void cSkinElchiDisplayChannel::Flush(void)
{
if (!noflush) {
#ifdef DEBUG
  isyslog("skinelchi: cSkinElchiDisplayChannel::Flush() is called");
#endif
  noflush = true;
    if (present) {
    	time_t presentstarttime = present->StartTime();
	if (withInfo) {
    	    time_t t = time(NULL);
	    if (t > presentstarttime) {
    		int seen = min(y9 - y6 - 1, int((y9 - y6) * double(t - presentstarttime) / present->Duration()));
		if (seen < 0)
		    seen = 0;
		if (seen != lastSeen) {
#ifdef DEBUG
		    isyslog("skinelchi: show seenbar in DisplayChannel::Flush()");
#endif
		    osd->DrawRectangle(x4 + Gap, y6 + seen, x4 + Gap + ScrollWidth - 1, y9 - 1, Theme.Color(clrChannelTimebarRest));
    		    osd->DrawRectangle(x4 + Gap, y6, x4 + Gap + ScrollWidth - 1, y6 + seen - 1, Theme.Color(clrChannelTimebarSeen));
    		    lastSeen = seen;
		    changed = true;
		    }
#ifdef DEBUG
		else
		    isyslog("skinelchi: skip show seenbar in DisplayChannel::Flush()");
#endif
		}
	    }
	time_t presentvps = present->Vps();
    	if((presentvps)!=oldpresentvps) {
	    if (presentvps != 0) {
		int d = 3;
		int x = setchannel_x;
		x -= bmVps.Width() + d;
		bool vps = (presentvps != presentstarttime);
		if (oldvps != vps || oldpresentvps == -1) {
#ifdef DEBUG
		    isyslog("skinelchi: show VPS-symbol in DisplayChannel::Flush()");
#endif
		    osd->DrawBitmap(x, y4 + (y6 - y4 - bmVps.Height()) / 2, bmVps, Theme.Color(vps ? clrChannelSymbolOn:clrChannelSymbolOff), bg);
		    changed = true;
		    oldvps = vps;
		    }
#ifdef DEBUG
		else
		    isyslog("skinelchi: skip show VPS-symbol in DisplayChannel::Flush()");
#endif
    		}
    	    oldpresentvps = presentvps;
	    }
	}
    else {
	if (withInfo) {
	    if (lastSeen) {
#ifdef DEBUG
		isyslog("skinelchi: clear seenbar in DisplayChannel::Flush()");
#endif
		lastSeen = 0;
		osd->DrawRectangle(x4 + Gap, y6, x4 + Gap + ScrollWidth - 1, y9 - 1, Theme.Color(clrChannelEpgTitleBg));
		changed = true;
		}
#ifdef DEBUG
	    else
		isyslog("skinelchi: skip clear seenbar in DisplayChannel::Flush()");
#endif
	    }
	oldpresentvps = -1;
	}
    if (!message) {
#ifdef VIDEOSTATUS
    if (fd_video >= 0) {
	if (video) {
	    int videoformat = -1;
	    char *videoformatchar = NULL;
	    char *tempvideo = NULL;
	    if (ioctl(fd_video, VIDEO_GET_SIZE, &vs) == 0) {
		switch (vs.aspect_ratio) {
		    case VIDEO_FORMAT_4_3: 
			videoformat = 0;
			asprintf(&tempvideo, "%d*%d %s", vs.w, vs.h, "4:3");
			break;
		    case VIDEO_FORMAT_16_9:
			videoformat = 1;
			asprintf(&tempvideo, "%d*%d %s", vs.w, vs.h, "16:9");
			break;
		    case VIDEO_FORMAT_221_1:
			videoformat = 2;
		        asprintf(&tempvideo, "%d*%d %s", vs.w, vs.h, "2.21:1");
		    }
		}
	    if (videoformat != -1) {
	    	if (ioctl(fd_video, VIDEO_GET_STATUS, &videostatus) == 0) {
		    cBitmap *bm = NULL;
		    switch (
#ifdef DVB_OLDER_20050331
videostatus.video_format
#else
videostatus.display_format
#endif
) {
			case 
#ifdef DVB_OLDER_20050331
VIDEO_FORMAT_4_3
#else
VIDEO_PAN_SCAN
#endif
: {
			    switch (videoformat) {
				case 0: /*4:3???*/
				    bm = Setup.VideoFormat ? &bmar16943 : &bmar43na;
				    break;
				case 1: /*16:9???*/
#if VDRVERSNUM >= 10322
#ifdef DVB_OLDER_20050331
		    		    if (Setup.VideoDisplayFormat != 0 && SkinElchiSetup.switchvideodisplayformat) /* must "check", VIDEO_FORMAT really show VIDEO_DISPLAY_FORMAT? */
				        cDevice::PrimaryDevice()->SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
#endif
#endif
				    bm = Setup.VideoFormat ? &bmar169na : &bmar43169ps;
				    break;
				case 2: /*2.21:1*/
				    bm = Setup.VideoFormat ? &bmar169ps : &bmar432211ps;
				}
			    break;
			    }
			case 
#ifdef DVB_OLDER_20050331
VIDEO_FORMAT_16_9
#else
VIDEO_LETTER_BOX
#endif
: {
			    switch (videoformat) {
				case 0: /*4:3???*/
				    bm = Setup.VideoFormat ? &bmar16943 : &bmar43na;
				    break;
				case 1: /*16:9???*/
#if VDRVERSNUM >= 10322
#ifdef DVB_OLDER_20050331
				    if (Setup.VideoDisplayFormat != 1 && SkinElchiSetup.switchvideodisplayformat) /* must "check", VIDEO_FORMAT really show VIDEO_DISPLAY_FORMAT? */
				        cDevice::PrimaryDevice()->SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
#endif
#endif
				    bm = Setup.VideoFormat ? &bmar169na : &bmar43169vc;
				    break;
				case 2: /*2.21:1*/
				    bm = Setup.VideoFormat ? &bmar169vc : &bmar432211vc;
				}
			    break;
			    }
			case 
#ifdef DVB_OLDER_20050331
VIDEO_FORMAT_221_1
#else
VIDEO_CENTER_CUT_OUT
#endif
: {
			    switch (videoformat) {
				case 0: /*4:3???*/
				    bm = Setup.VideoFormat ? &bmar16943 : &bmar43na;/*4:3 on 16:9*/
				    break;
				case 1: /*16:9???*/
				    bm = Setup.VideoFormat ? &bmar169na : &bmar43169ps;
				    break;
				case 2: /*2.21:1*/
#if VDRVERSNUM >= 10322
#ifdef DVB_OLDER_20050331
		    		    if (Setup.VideoDisplayFormat != 2 && SkinElchiSetup.switchvideodisplayformat) /* must "check", VIDEO_FORMAT really show VIDEO_DISPLAY_FORMAT? */
		    		        cDevice::PrimaryDevice()->SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
#endif
#endif
				    bm = Setup.VideoFormat ? &bmar169vc : &bmar432211psvc;
				}
			    }
			}
		    if (!showVolume && !message) {
		    if (bm) {
			if (bm != oldbm || changed) {
			    osd->DrawBitmap(x9 + lh, y4 + (y6 - y4 - bm->Height()) / 2, *bm, Theme.Color(clrChannelSymbolOn), bg);
			    oldbm = bm;
			    changed = true;
#ifdef DEBUG
			    isyslog("skinelchi: show videodisplayformatsymbol in DisplayChannel::Flush()");
#endif
			    }
#ifdef DEBUG
			else
			    isyslog("skinelchi: skip show videodisplayformatsymbol in DisplayChannel::Flush()");
#endif
			}
		    else {
			if (oldbm) {
			    osd->DrawRectangle(x9 + lh, y4, x9 + lh + bmar169na.Width() - 1, y6 - 1, bg);
			    oldbm = NULL;
			    changed = true;
#ifdef DEBUG
			    isyslog("skinelchi: erase  videodisplayformatsymbol in DisplayChannel::Flush()");
#endif
			    }
#ifdef DEBUG
			else
			    isyslog("skinelchi: skip erase  videodisplayformatsymbol in DisplayChannel::Flush()");
#endif
			}
		      }
		    }
		asprintf(&videoformatchar, " %s (%d:%d:%d:%d:%d)", tempvideo, (int)videostatus.video_blank, (int)videostatus.play_state, (int)videostatus.stream_source, (int)(videostatus.video_format), (int)videostatus.display_format);
		if (tempvideo)
		    free(tempvideo);
		}
	    int w = smallfont->Width(videoformatchar);
	    int x = x9 - x3;
	    if (w > x)
	        w = x;
	    if (w != oldvideow) {
		if (w < oldvideow) {
#ifdef DEBUG
		    isyslog("skinelchi: erase videoformatchar in DisplayChannel::Flush()");
#endif
		    osd->DrawRectangle(x9 - oldvideow, y4 + lineOffset, x9 - w - 1, y4 + lineOffset + slh - 1, Theme.Color(clrChannelNameBg));
		    changed = true;
		    }
#ifdef DEBUG
		else    
		    isyslog("skinelchi: skip erase videoformatchar in DisplayChannel::Flush()");
#endif
		oldvideow = w;
		}
	    if (videoformatchar) {
		if (strcmp(oldvideoformatchar, videoformatchar)) {
#ifdef DEBUG
		    isyslog("skinelchi: show videoformatchar in DisplayChannel::Flush()");
#endif
		    osd->DrawText(x9 - w, y4 + lineOffset, videoformatchar, Theme.Color(clrChannelNameFg), Theme.Color(clrChannelNameBg), smallfont, w);
		    if (oldvideoformatchar)
			free(oldvideoformatchar);
		    asprintf(&oldvideoformatchar, "%s", videoformatchar);
		    changed = true;
		    }
#ifdef DEBUG
		else
		    isyslog("skinelchi: skip show videoformatchar in DisplayChannel::Flush()");
#endif
		free(videoformatchar);
		}
    	    }
        }
#endif
	cDevice *Device = cDevice::PrimaryDevice();
	const tTrackId *Track = Device->GetTrack(Device->GetCurrentAudioTrack());
	int w = 0;
	if (Track) {
	    char *audiostring = NULL;
	    int numaudiotracks = Device->NumAudioTracks();
	    if (numaudiotracks) {
		if (VdrStatus.AudioTrack) {
		    asprintf(&audiostring, "%s", VdrStatus.AudioTrack);
		    if (numaudiotracks > 1)
			asprintf(&audiostring, "%s %d/%d", audiostring, VdrStatus.AudioTrackIndex, numaudiotracks);
		    }
		else {
		    asprintf(&audiostring, "%s", Track->description);
		    w = smallfont->Width(audiostring);
		    if (!w)
	    		asprintf(&audiostring, "%s", Track->language);
		    if (numaudiotracks > 1)
			asprintf(&audiostring, "%s /%d", audiostring, numaudiotracks);
		    }
		if (VdrStatus.AudioChannel)
		    asprintf(&audiostring, "%s (%s)",audiostring, VdrStatus.AudioChannel);
		}
	    else
		asprintf(&audiostring, "%s", tr("No audio available!"));
	    if (VdrStatus.RecordingNumbers) {
		for (int i = 0; i < MAXDEVICES; i++)
		    if (VdrStatus.RecordingNumber[i] > 0) {
			if (VdrStatus.RecordingNumber[i] == 1)
			    asprintf(&audiostring, "%s | DVB%d: %d %s",audiostring, i + 1, VdrStatus.RecordingNumber[i], tr("Recording"));
			else
			    asprintf(&audiostring, "%s | DVB%d: %d %s",audiostring, i + 1, VdrStatus.RecordingNumber[i], tr("Recordings"));
		    }
		}
	    w = smallfont->Width(audiostring);
	    if (strcmp(AudioStringScroll.Text, audiostring)) {
#ifdef DEBUG
		isyslog("skinelchi: show audiostring - DisplayChannel::Flush()");
#endif
		if (AudioStringScroll.SetText(audiostring)) {//for scrolling
		    AudioStringScroll.pos = 0;
		    AudioStringScroll.dir = -1;
		    AudioStringScroll.delay = 5;
		    AudioStringScroll.Time.Set();
		    osd->DrawText(AudioStringScroll.xOff, AudioStringScroll.yOff, AudioStringScroll.Text, AudioStringScroll.Fg, AudioStringScroll.Bg, AudioStringScroll.Font, AudioStringScroll.DestWidth);
		    }
		else {
		    AudioStringScroll.dir = 0;
		    osd->DrawText(x10 - w, AudioStringScroll.yOff, AudioStringScroll.Text, AudioStringScroll.Fg, AudioStringScroll.Bg, AudioStringScroll.Font, w);
		    }
		changed = true;
		}
#ifdef DEBUG
	    else
		isyslog("skinelchi: skip show audiostring - DisplayChannel::Flush()");
#endif
	    free(audiostring);
	    }
	if (oldaudiow != w) {
	    if (oldaudiow > w) {
		int x = x10 - x7;
		if (w < x) {
		    osd->DrawRectangle(x10 - ((oldaudiow > x) ? x : oldaudiow), y1, x10 - w - 1, y3 - 1, bg);
		    changed = true;
#ifdef DEBUG
		    isyslog("skinelchi: erase unused audiostring - DisplayChannel::Flush()");
#endif
		    }
		}
#ifdef DEBUG
	    else
		isyslog("skinelchi: no unused audiostring - DisplayChannel::Flush()");
#endif
	    oldaudiow = w;
	    }
#ifdef DEBUG
	else
	    isyslog("skinelchi: skip erase unused audiostring - DisplayChannel::Flush()");
#endif
	}
    if (AudioStringScroll.dir) {//for scrolling
	if (AudioStringScroll.DrawNeeded()) {
	    for (int i = 0; i < AudioStringScroll.DestWidth; i++) {
		for (int j = 0; j < AudioStringScroll.Height(); j++)
		    osd->DrawPixel(AudioStringScroll.xOff + i, AudioStringScroll.yOff + j, AudioStringScroll.bmp->Color(*(AudioStringScroll.bmp->Data(i + AudioStringScroll.pos, j))));
		}
	    changed = true;
	    }
	}

    bool rectemp = cRecordControls::Active();
    if (!showVolume && !message && rec != rectemp) {
	rec = rectemp;
	osd->DrawBitmap(x10 - bmRecording.Width(), y4 + (y6 - y4 - bmRecording.Height()) / 2, bmRecording, Theme.Color(rec ? clrChannelSymbolRecFg : clrChannelSymbolOff), rec ? Theme.Color(clrChannelSymbolRecBg) : bg);
	changed = true;
	}

    if (VdrStatus.VolumeChange != volumechange) {
	volumechange = VdrStatus.VolumeChange;
	oldvolume = VdrStatus.Volume;
	int w = x10;
	int y = y4 + 4;
	int h = y6 - 5;
	if (!showVolume) {
	    if (olddate)
		free(olddate);
	    asprintf(&olddate, " ");
	    osd->SaveRegion(logox, y4, x11 - 1, y6 - 1);
	    osd->DrawText(x3, y1, tr("Volume "), Theme.Color(clrChannelDateFg), Theme.Color(clrChannelDateBg), font, x6 - x3);
	    osd->DrawRectangle(x3, y4, x9 + lh / 2 - 1, y - 1, bg);
	    osd->DrawRectangle(x3, h + 1, x9 + lh / 2 - 1, y6 - 1, bg);
	    if (withInfo) {
		if (SkinElchiSetup.logo)
		    osd->DrawRectangle(x1, y5, logox-1, y6 - 1, bg);
		osd->DrawRectangle(logox, y5, x3-1, y6 - 1, bg);
		osd->DrawRectangle(x10, y4, x11 - 1, y6 - 1, bg);
		}
	    else {
		if (SkinElchiSetup.logo) {
	    	    osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
	    	    osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, bg, 3);
		    osd->DrawRectangle(x2, y5, logox - 1, y6 - 1, bg);
	    	    osd->DrawRectangle(logox, y5, x3 - 1, y6 - 1, bg);
	    	    }
		else {
	    	    osd->DrawRectangle(logox, y5, x3 - 1, y6 - 1, clrTransparent);
	    	    osd->DrawEllipse  (logox, y5, x3 - 1, y6 - 1, bg, 3);
	    	    }
		osd->DrawRectangle(x10, y5, x11 - 1, y6 - 1, clrTransparent);
		osd->DrawEllipse  (x10, y5, x11 - 1, y6 - 1, bg, 4);
		}
	    osd->DrawRectangle(x10, y4, x11 - 1, y5 - 1, bg);
	    osd->DrawRectangle(logox, y4, x3-1, y5 - 1, bg);
	    if (SkinElchiSetup.logo) {
		osd->DrawRectangle(x1, logoy, logox-1, y5 - 1, bg);
    		osd->DrawRectangle(logox, y0 + 2, logox + 2 - 1, logoy + 2 - 1, bg);
    		osd->DrawRectangle(x1, logoy, logox - 1, logoy + 2 - 1, bg);
		}
    	    showVolume = true;
	    }
	int p = (w - x3) * oldvolume / 255;
	int xp = x3 + p;
	osd->DrawRectangle(x3, y, xp - 1, h, Theme.Color(clrVolumeBarUpper));
	osd->DrawRectangle(xp, y, w - 1, h, Theme.Color(clrVolumeBarLower));
	changed = true;
	volume.Set(1000);
	}
    else { 
	if (showVolume) {
	    if(volume.TimedOut()) {
		osd->DrawText(x3, y1, number, Theme.Color(clrChannelNumberFg), Theme.Color(clrChannelDateBg), font, font->Width(tr("Volume ")));
		if (SkinElchiSetup.logo) {
         	    if (withInfo)
			osd->DrawRectangle(x1, logoy, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));
		    else {
			osd->DrawRectangle(x1, logoy, logox - 1, y5 - 1, Theme.Color(clrChannelNameBg));
    			osd->DrawRectangle(x2, y5, logox - 1, y6 - 1, Theme.Color(clrChannelNameBg));
			osd->DrawRectangle(x1, y5, x2 - 1, y6 - 1, clrTransparent);
			osd->DrawEllipse  (x1, y5, x2 - 1, y6 - 1, Theme.Color(clrChannelNameBg), 3);
			}
    		    osd->DrawRectangle(logox, y0 + 2, logox + 2 - 1, logoy + 2 - 1, bg);
    		    osd->DrawRectangle(x1, logoy, logox - 1, logoy + 2 - 1, bg);
		    }
		osd->RestoreRegion();
		showVolume = false;
		changed = true;
		}
	    }
	else {
	cString date = DayDateTime();
	    if (strcmp(olddate, date)) {
#ifdef DEBUG
		isyslog("skinelchi: show DayDateTime in DisplayChannel::Flush()");
#endif
		changed = true;
		if (olddate)
		    free(olddate);
		asprintf(&olddate, date);
		osd->DrawText(x6 - smallfont->Width(date), y1 + lineOffset, date, Theme.Color(clrChannelDateFg), Theme.Color(clrChannelDateBg), smallfont);
		}
#ifdef DEBUG
	    else
		isyslog("skinelchi: skip show DayDateTime in DisplayChannel::Flush()");
#endif
	    if (!message && ChannelNameScroll.dir) {//for scrolling
		if (ChannelNameScroll.DrawNeeded()) {
		    for (int i = 0; i < ChannelNameScroll.DestWidth; i++) {
			for (int j = 0; j < ChannelNameScroll.Height(); j++)
			    osd->DrawPixel(ChannelNameScroll.xOff + i, ChannelNameScroll.yOff + j, ChannelNameScroll.bmp->Color(*(ChannelNameScroll.bmp->Data(i + ChannelNameScroll.pos, j))));
		        }
		    changed = true;
		    }
	        }
	    }
	}

    if (EventScroll.dir) {//for scrolling
	if (EventScroll.DrawNeeded()) {
	    for (int i = 0; i < EventScroll.DestWidth; i++) {
		for (int j = 0; j < EventScroll.Height(); j++)
		    osd->DrawPixel(EventScroll.xOff + i, EventScroll.yOff + j, EventScroll.bmp->Color(*(EventScroll.bmp->Data(i + EventScroll.pos, j))));
	        }
	    changed = true;
	    }
	}

    if (EventShortScroll.dir) {//for scrolling
	if (EventShortScroll.DrawNeeded()) {
	    for (int i = 0; i < EventShortScroll.DestWidth; i++) {
		for (int j = 0; j < EventShortScroll.Height(); j++)
		    osd->DrawPixel(EventShortScroll.xOff + i, EventShortScroll.yOff + j, EventShortScroll.bmp->Color(*(EventShortScroll.bmp->Data(i + EventShortScroll.pos, j))));
	        }
	    changed = true;
	    }
	}

    if (NextEventScroll.dir) {//for scrolling
	if (NextEventScroll.DrawNeeded()) {
	    for (int i = 0; i < NextEventScroll.DestWidth; i++) {
		for (int j = 0; j < NextEventScroll.Height(); j++)
		    osd->DrawPixel(NextEventScroll.xOff + i, NextEventScroll.yOff + j, NextEventScroll.bmp->Color(*(NextEventScroll.bmp->Data(i + NextEventScroll.pos, j))));
	        }
	    changed = true;
	    }
	}

    if (NextEventShortScroll.dir) {//for scrolling
	if (NextEventShortScroll.DrawNeeded()) {
	    for (int i = 0; i < NextEventShortScroll.DestWidth; i++) {
		for (int j = 0; j < NextEventShortScroll.Height(); j++)
		    osd->DrawPixel(NextEventShortScroll.xOff + i, NextEventShortScroll.yOff + j, NextEventShortScroll.bmp->Color(*(NextEventShortScroll.bmp->Data(i + NextEventShortScroll.pos, j))));
	        }
	    changed = true;
	    }
	}

    if (changed) {
#ifdef DEBUG
	isyslog("skinelchi: DisplayChannel::Flush()");
#endif
	osd->Flush();
	changed = false;
	}
#ifdef DEBUG
    else
	isyslog("skinelchi: skip DisplayChannel::Flush()");
#endif
    noflush = false;
  }
#ifdef DEBUG
else
  isyslog("skinelchi: skip cSkinElchiDisplayChannel::Flush() because cSkinElchiDisplayChannel::Flush() is active");
#endif
}
